// (c) 2015, SAS Institute Inc.
sap.ui.define([
    "jquery.sap.global",
    "sas/hc/m/Button"
], function(jQuery, Button) {
    "use strict";

    // icon path constants
    var SELECTED_ICON_PATH = sas.icons.HC.FAVORITEDINDICATOR,
        NOT_SELECTED_ICON_PATH = sas.icons.HC.FAVORITENOTSELECTED;

    /**
     * Constructor for a new FavoritesButton.
     *
     * Handles state with the "selected" property.
     * Depending on state, a different icon will show.
     *
     * @class
     * @name sas.hc.m.objectInspector.FavoritesButton
     * @extends sas.hc.m.Button
     * @version 207001.0.1.20170511082246_f0htmcm27p
     * @author Jonathan Brink
     * @constructor
     * @public
     */
    var FavoritesButton = Button.extend("sas.hc.m.objectInspector.FavoritesButton", /** @lends sas.hc.m.objectInspector.FavoritesButton.prototype */ {
        metadata: {
            library: "sas.hc.m",
            properties: {
                selected: {type: "boolean", defaultValue: false, group: "Appearance"}
            }
        },
        renderer: {
        }
    });

    /**
     * init lifecycle callback.
     * @private
     */
    FavoritesButton.prototype.init = function() {
        // set default icon
        this.setSelected(!!this.getSelected());
    };

    /**
     * tap event handler
     * When button is tapped/clicked, toggle it's "selected" state
     * @public
     */
    FavoritesButton.prototype.ontap = function() {
        if (Button.prototype.ontap) {
            Button.prototype.ontap.apply(this, arguments);
        }
        this.toggleSelected();
    };

    /**
     * "selected" property setter override
     * prevent invalidation
     * manages icon property value
     * @param {boolean} bSelected value of selected
     * @public
     */
    FavoritesButton.prototype.setSelected = function(bSelected) {
        this.setProperty('selected', bSelected, true);
        if (bSelected === true) {
            this.setIcon(SELECTED_ICON_PATH);
        } else {
            this.setIcon(NOT_SELECTED_ICON_PATH);
        }
    };

    /**
     * toggle the "selected" property
     * @public
     */
    FavoritesButton.prototype.toggleSelected = function() {
        var bNewSelectedState = true;
        if (this.getSelected() === true) {
            bNewSelectedState = false;
        }
        this.setSelected(bNewSelectedState);
    };

    return FavoritesButton;
}, /* bExport= */ true);
