// (c) 2015, SAS Institute Inc.
sap.ui.define([
    "jquery.sap.global",
    "sap/ui/core/Control"
], function(jQuery, Control) {
    "use strict";

    /**
     * Renders a simply name-value pair html table.
     *
     * Support for showing ellipses after the name values.
     *
     * @class
     * @name sas.hc.m.objectInspector.SimplePairTable
     * @extends sap.ui.core.Control
     * @author Jonathan Brink
     * @version 207001.0.1.20170511082246_f0htmcm27p
     * @constructor
     * @private
     */
    var SimplePairTable = Control.extend("sas.hc.m.objectInspector.SimplePairTable", /** @lends sas.hc.m.objectInspector.SimplePairTable.prototype */ {
        metadata: {
            library: "sas.hc.m",
            properties: {
                /**
                 * Array of object name-value pairs.
                 * Example: [{name: '', value: ''}, {name: '', value: ''}]
                 */
                data: {type: "object", defaultValue: [], group: "Data"}
            }
        },
        renderer: {
            /**
             * Render the _SimplePairTable
             * @param {object} oRenderManager The render manager
             * @param {object} oControl The _SimplePairTable control
             * @public
             */
            render: function(oRenderManager, oControl) {
                oRenderManager.write('<div');
                oRenderManager.writeControlData(oControl);
                oRenderManager.addClass("sasSimplePairTable");
                oRenderManager.writeClasses();
                oRenderManager.write('>');
                this.renderTable(oRenderManager, oControl);
                oRenderManager.write('</div>');
            },

            /**
             * Render html table from the _SimplePairTable's data aggregation
             * @public
             * @param {object} oRenderManager openUi5 render manager
             * @param {object} oControl The _SimplePairTable
             */
            renderTable: function(oRenderManager, oControl) {
                var aData = oControl.getData();
                oRenderManager.write('<table>');
                oRenderManager.write('<tbody>');
                aData.forEach(function(oData) {
                    oRenderManager.write('<tr>');
                    oRenderManager.write('<td>' + oData.name);
                    oRenderManager.write('</td>');
                    oRenderManager.write('<td>' + oData.value + '</td>');
                    oRenderManager.write('</tr>');
                });
                oRenderManager.write('</tbody>');
                oRenderManager.write('</table>');
            }
        }
    });

    /**
     * onBeforeRendering lifecycle callback.
     * @private
     */
    SimplePairTable.prototype.onAfterRendering = function() {
        var oData = this.getData(),
            oDomRef = this.getDomRef(),
            $domTable = jQuery('table', oDomRef),
            iMinHeight = 100;

        if (oData.length <= 2) {
            iMinHeight = 50;
        }

        $domTable.css('min-height', iMinHeight + 'px');
    };

    return SimplePairTable;
}, /* bExport= */ true);
